package com.ruoyi.ss.device.service;

import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceCountVO;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * 设备Service接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface ISmDeviceService
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备集合
     */
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceQuery smDevice);

    /**
     * 新增设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int insertSmDevice(SmDevice smDevice);

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    public int updateSmDevice(SmDevice smDevice);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键集合
     * @return 结果
     */
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 条件查询数量
     * @param dto 查询条件
     * @return 结果
     */
    int selectCount(SmDeviceQuery dto);

    /**
     * 查询指定店铺下的设备列表
     * @param storeId
     */
    List<SmDeviceVo> selectSmDeviceByStoreId(Long storeId);

    /**
     * 增加时间
     * @param deviceId 设备id
     * @param num 电量
     */
    boolean addTime(Long deviceId, BigDecimal num);

    /**
     * 逻辑删除
     */
    boolean logicDel(List<Long> deviceIds);

    /**
     * 通过型号id列表，查询设备列表，并分组
     * @param modelIds 型号id列表
     */
    Map<Long, List<SmDeviceVo>> selectGroupByModelIds(List<Long> modelIds);

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    boolean changePowerStatus(Long deviceId, DevicePowerStatus status);

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    void pullDeviceInfo(List<Long> deviceIds);

    /**
     * 通过mac绑定设备
     * 设备第一次绑定为商户
     * 其他时候绑定为租户
     * @param storeId 店铺ID
     * @param deviceNo 设备编号（SN）
     */
    boolean bind(Long storeId, String deviceNo);

    /**
     * 商户解绑设备
     * @param deviceId 设备id
     */
    boolean unbindStore(Long deviceId);

    /**
     * 通过mac查询设备
     * @param mac mac号
     * @return 设备信息
     */
    SmDeviceVo selectByMac(String mac);

    /**
     * 归零电表
     * @param deviceId 设备id
     */
    boolean reset(Long deviceId);

    /**
     * 设备是否已经被绑定
     * @param mac
     * @return
     */
    boolean isBindByMac(String mac);

    /**
     * ids查询列表
     */
    List<SmDeviceVo> selectListByIds(List<Long> ids);

    /**
     * 查询各店铺的设备数量,并分组
     * @param storeIds 店铺id
     */
    Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds);

    /**
     * 条件查询数量，并分组
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    <K> Map<K, Integer> selectCommonCountMap(SmDeviceQuery query, Function<? super SmDeviceCountVO, ? extends K> keyMapper );

    /**
     * 获取设备，包含套餐列表
     * @param deviceNo 设备编码
     * @return
     */
    SmDeviceVo getDeviceWithSuitList(String deviceNo);

    /**
     * 根据设备编号查询
     */
    SmDeviceVo selectByDeviceNo(String deviceNo);

    /**
     * 初始设备录入
     * @param smDevice
     */
    int addInitDevice(SmDevice smDevice);

    /**
     * 设备绑定SN码
     * @param deviceId 设备ID
     * @param sn SN
     */
    int bindSn(Long deviceId, String sn);

    /**
     * 根据SN码查询设备是否已经绑定
     * @param deviceNo
     * @return
     */
    boolean isBindBySn(String deviceNo);
}
