package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.ss.device.domain.SmDevice;
import com.ruoyi.ss.device.domain.SmDeviceCountVO;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.vo.SmDeviceVo;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.mapper.SmDeviceMapper;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.deviceBindRecord.service.ISmDeviceBindRecordService;
import com.ruoyi.ss.resetRecord.domain.SmResetRecord;
import com.ruoyi.ss.resetRecord.service.ISmResetRecordService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.service.IotService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class SmDeviceServiceImpl implements ISmDeviceService
{
    @Autowired
    private SmDeviceMapper smDeviceMapper;

    @Autowired
    private IotService iotService;

    @Autowired
    private ISmDeviceBindRecordService smDeviceBindRecordService;

    @Autowired
    private ISmResetRecordService smResetRecordService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IStoreService storeService;

    @Value("${debug}")
    private Boolean debug;

    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public SmDeviceVo selectSmDeviceByDeviceId(Long deviceId)
    {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        if (device == null) {
            return null;
        }
        assembleDeviceInfo(Collections.singletonList(device));
        return device;
    }

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备
     */
    @Override
    public List<SmDeviceVo> selectSmDeviceList(SmDeviceQuery smDevice)
    {
        smDevice.setDeleted(false);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(smDevice);
        assembleDeviceInfo(list);
        return list;
    }

    /**
     * 拼接设备信息
     * @param list 设备列表
     */
    private void assembleDeviceInfo(List<SmDeviceVo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 获取当前设备数据
        List<String> deviceNames = list.stream().map(SmDeviceVo::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(deviceNames)) {
            return;
        }
        Map<String, IotDeviceInfo> deviceInfoMap = iotService.getDeviceInfo(deviceNames)
                .stream().collect(Collectors.toMap(IotDeviceInfo::getMac, item -> item));

        // 拼接数据
        for (SmDeviceVo device : list) {
            if (StringUtils.hasText(device.getMac())) {
                // 在线状态
                device.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                // 其他信息
                IotDeviceInfo deviceInfo = deviceInfoMap.get(device.getMac());
                if (deviceInfo != null) {
                    device.setElectricity(deviceInfo.getA());
                    device.setVoltage(deviceInfo.getV());
                    device.setRealTimePower(deviceInfo.getP());
                    device.setTotalElectriQuantity(deviceInfo.getW());  // 电量
                }
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
            }
        }
    }

    /**
     * 新增设备
     * F
     * @param data 设备
     * @return 结果
     */
    @Override
    public int insertSmDevice(SmDevice data)
    {
        this.validate(data, false);
        data.setCreateTime(DateUtils.getNowDate());
        data.setStatus(DeviceStatus.NORMAL.getStatus());
        return smDeviceMapper.insertSmDevice(data);
    }

    /**
     * 修改设备
     *
     * @param smDevice 设备
     * @return 结果
     */
    @Override
    @Transactional
    public int updateSmDevice(SmDevice smDevice)
    {
        // 校验
        this.validate(smDevice, true);

        // 修改数据库
        smDevice.setUpdateTime(DateUtils.getNowDate());
        int updateCount = smDeviceMapper.updateSmDevice(smDevice);

        // 设置物联网设备信息
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(smDevice.getDeviceId());
        if (!StringUtils.isBlank(device.getMac())) {
            boolean updateDevice = iotService.updateDevice(device);
            ServiceUtil.assertion(!updateDevice, "设备修改失败，请刷新后重试");
        }

        return updateCount;
    }

    private void validate(SmDevice smDevice, boolean update) {
        // 判断是否有权限修改
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (update) {
            ServiceUtil.assertion(smDevice.getDeviceId() == null, "参数错误：设备id不允许为空");
            SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(smDevice.getDeviceId());
            ServiceUtil.assertion(device == null || device.getDeleted(), "设备" + smDevice.getDeviceId() + "不存在");
            if (LoginType.FRONT.equals(loginUser.getLoginType())) {
                ServiceUtil.assertion(!Objects.equals(device.getUserId(), loginUser.getUserId()), "这不是您的设备");
            }
        }
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds)
    {
        this.validatePreDelete(Arrays.asList(deviceIds));
        return smDeviceMapper.deleteSmDeviceByDeviceIds(deviceIds);
    }

    /**
     * 真正的删除校验
     */
    private void validatePreDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(true);
        Map<Long, SmDeviceVo> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            SmDeviceVo device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }
    }

    /**
     * 逻辑删除校验
     */
    private void validatePreLogicDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");

        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(false);
        // 前台用户，判断是不是他的设备
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            dto.setUserId(SecurityUtils.getUserId());
        }
        Map<Long, SmDeviceVo> deviceMap = selectMap(dto);

        deviceAssembler.assembleTenant(new ArrayList<>(deviceMap.values()));
        for (Long deviceId : deviceIds) {
            SmDeviceVo device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
            ServiceUtil.assertion(!CollectionUtils.isEmpty(device.getTenantIds()), String.format("设备“%s”还有租户，请解绑后再进行删除", device.getDeviceName()));
        }

    }

    /**
     * 条件查询列表，并转为Map
     */
    private Map<Long, SmDeviceVo> selectMap(SmDeviceQuery dto) {
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(SmDeviceVo::getDeviceId, item -> item));
    }

    @Override
    public int selectCount(SmDeviceQuery dto) {
        return smDeviceMapper.selectCount(dto);
    }

    @Override
    public List<SmDeviceVo> selectSmDeviceByStoreId(Long storeId) {
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setStoreId(storeId);
        dto.setDeleted(false);
        return smDeviceMapper.selectSmDeviceList(dto);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public boolean addTime(Long deviceId, BigDecimal num) {
        ServiceUtil.assertion(num.compareTo(BigDecimal.ZERO) < 0, "增加的时长不允许小于0");

        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备MAC号为空");
        ServiceUtil.assertion(DeviceStatus.FIXING.getStatus().equals(device.getStatus()), "设备正在维修中，无法使用");

        // 更新数据库时长
        int updateCount = smDeviceMapper.addTime(deviceId, num);
        ServiceUtil.assertion(updateCount != 1, "增加时长失败，请刷新后重试");

        // 修改状态为使用中
        changeStatus(deviceId, DeviceStatus.USING);

        // 物联网设备增加时长
        boolean rechargeResult = iotService.recharge(device.getMac(), num);
        ServiceUtil.assertion(!rechargeResult, "设备充值失败");

        // 拉取设备信息
        this.pullDeviceInfoAsync(Collections.singletonList(deviceId), 3, TimeUnit.SECONDS);

        return true;
    }

    private int changeStatus(Long deviceId, DeviceStatus status) {
        SmDevice device = new SmDevice();
        device.setDeviceId(deviceId);
        device.setStatus(status.getStatus());
        return smDeviceMapper.updateSmDevice(device);
    }

    /**
     * 获取租户设备列表
     * @param tenantId 租户id
     * @return
     */
    private List<SmDeviceVo> selectTenantDeviceList(Long tenantId) {
        if (tenantId == null) {
            return Collections.emptyList();
        }
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeleted(false);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        assembleDeviceInfo(list);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    @Override
    public boolean logicDel(List<Long> deviceIds) {
        this.validatePreLogicDelete(deviceIds);
        return smDeviceMapper.logicDel(deviceIds) > 0;
    }

    @Override
    public Map<Long, List<SmDeviceVo>> selectGroupByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeleted(false);
        dto.setModelIds(modelIds);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(SmDeviceVo::getModelId));
    }

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    @Override
    public boolean changePowerStatus(Long deviceId, DevicePowerStatus status) {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!StringUtils.hasText(device.getMac()), "设备Mac号为空");

        deviceAssembler.assembleTenant(Collections.singletonList(device));
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的商户，无法进行该操作" );

        if (DevicePowerStatus.ON.equals(status)) {
            iotService.open(device.getMac());
        } else if (DevicePowerStatus.OFF.equals(status)) {
            iotService.close(device.getMac());
        } else {
            throw new ServiceException("不支持的操作");
        }

        // 修改数据库通断电状态
        SmDevice changePowerForm = new SmDevice();
        changePowerForm.setDeviceId(deviceId);
        changePowerForm.setPowerStatus(status.getStatus());
        smDeviceMapper.updateSmDevice(changePowerForm);

        return true;
    }

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    @Override
    @SneakyThrows
    public void pullDeviceInfo(List<Long> deviceIds) {
        // 查询设备列表
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeviceIds(deviceIds);
        List<SmDeviceVo> list = smDeviceMapper.selectSimpleList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        // 获取数据点信息
        List<String> deviceNames = list.stream().map(SmDeviceVo::getMac).filter(StringUtils::hasText).collect(Collectors.toList());
        List<IotDeviceInfo> deviceInfos = iotService.getDeviceInfo(deviceNames);

        // 更新数据
        for (SmDeviceVo device : list) {
            try {
                IotDeviceInfo deviceInfo = deviceInfos.stream().filter(item -> Objects.equals(device.getMac(), item.getMac())).findFirst().orElse(null);
                SmDevice data = new SmDevice();
                data.setDeviceId(device.getDeviceId());
                if (deviceInfo != null) {
                    data.setLastPullTime(deviceInfo.getAt());
                    data.setTotalElectriQuantity(deviceInfo.getW());
                    data.setPowerStatus(deviceInfo.getS());
                }

                // 是否在线
                data.setOnlineStatus(iotService.getOnlineStatus(device.getMac()).getStatus());

                smDeviceMapper.updateSmDevice(data);
            } catch (Exception e) {
                log.error("更新设备信息失败，deviceId:{}", device.getDeviceId(), e);
            }
        }
    }

    /**
     * 异步延迟执行拉取设备信息
     * @param deviceIds 设备id列表
     * @param delay 延迟时间
     * @param unit 时间单位
     */
    private void pullDeviceInfoAsync(List<Long> deviceIds, long delay, TimeUnit unit) {
        scheduledExecutorService.schedule(() -> {
            log.info("异步拉取设备信息 start");
            this.pullDeviceInfo(deviceIds);
            log.info("异步拉取设备信息 end");
        }, delay, unit);
    }

    @Override
    @Transactional
    public boolean bind(Long storeId, String deviceNo) {
        // 判断设备是否存在
        SmDeviceVo device = selectByDeviceNo(deviceNo);
        ServiceUtil.assertion(device == null, "设备未录入");
        ServiceUtil.assertion(device.getStoreId() != null, "该设备已被绑定");

        // 查询店铺
        StoreVo store = storeService.selectSmStoreById(storeId);
        ServiceUtil.assertion(store == null, "店铺不存在");

        // 绑定店铺
        int updateCount = smDeviceMapper.bindStore(device.getDeviceId(), storeId);
        ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

        // 记录设备绑定
        scheduledExecutorService.schedule(() -> {
            smDeviceBindRecordService.record(store.getUserId(), device.getDeviceId());
        }, 0, TimeUnit.SECONDS);

        return true;
    }

    /**
     * 根据mac号查询数据
     * @param mac
     * @return
     */
    @Override
    public SmDeviceVo selectByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setMac(mac);
        List<SmDeviceVo> list = smDeviceMapper.selectSmDeviceList(dto);
        assembleDeviceInfo(list);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public boolean reset(Long deviceId) {
        ServiceUtil.assertion(deviceId == null, "设备id不能为空");

        LocalDateTime now = LocalDateTime.now();
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(!UserUtil.hasFrontUser(device.getUserId()), "您不是该设备的商户，无法进行该操作");
        ServiceUtil.assertion(device.getExpireTime() == null || device.getExpireTime().isBefore(now), "设备剩余时间不足，无需归零");

        // 归零时间，将过期时间设置为现在
        SmDevice form = new SmDevice();
        form.setDeviceId(deviceId);
        form.setExpireTime(now);
        form.setStatus(DeviceStatus.NORMAL.getStatus());
        smDeviceMapper.updateSmDevice(form);

        // TODO 物联网设备归零

        // 归零记录
        scheduledExecutorService.schedule(() -> {
            SmResetRecord reset = new SmResetRecord();
            reset.setDeviceId(deviceId);
            reset.setMchId(device.getUserId());
            Duration duration = Duration.between(now, device.getExpireTime());
            reset.setSurplusTime(new BigDecimal(duration.toMinutes()));
            smResetRecordService.insertSmResetRecord(reset);
        }, 0, TimeUnit.SECONDS);

        return true;
    }

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    @Override
    public boolean isBindByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return false;
        }
        SmDeviceVo device = smDeviceMapper.selectSimpleSmDeviceByMac(mac);
        return device != null && device.getStoreId() != null;
    }

    /**
     * 根据SN码查询设备是否已经绑定
     *
     * @param deviceNo
     * @return
     */
    @Override
    public boolean isBindBySn(String deviceNo) {
        if (StringUtils.isBlank(deviceNo)) {
            return false;
        }
        SmDeviceVo device = smDeviceMapper.selectSimpleSmDeviceBySn(deviceNo);
        return device != null && device.getStoreId() != null;
    }

    /**
     * ids查询列表
     *
     * @param ids
     */
    @Override
    public List<SmDeviceVo> selectListByIds(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        SmDeviceQuery dto = new SmDeviceQuery();
        dto.setDeviceIds(ids);
        return selectSmDeviceList(dto);
    }

    /**
     * 查询各店铺的设备数量,并店铺
     *
     * @param storeIds 店铺id
     */
    @Override
    public Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds) {
        if (CollectionUtils.isEmptyElement(storeIds)) {
            return Collections.emptyMap();
        }
        List<SmDeviceCountVO> list = smDeviceMapper.selectCountByStoreIds(storeIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(SmDeviceCountVO::getStoreId, SmDeviceCountVO::getCount));
    }

    /**
     * 条件查询数量，并店铺
     *
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    @Override
    public <K> Map<K, Integer> selectCommonCountMap(SmDeviceQuery query, Function<? super SmDeviceCountVO, ? extends K> keyMapper ) {
        if (query == null) {
            return Collections.emptyMap();
        }
        List<SmDeviceCountVO> list = smDeviceMapper.selectCommonCount(query);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, SmDeviceCountVO::getCount));
    }

    /**
     * 获取设备，包含套餐列表
     *
     * @param deviceSn
     * @return
     */
    @Override
    public SmDeviceVo getDeviceWithSuitList(String deviceSn) {
        if (deviceSn == null) {
            return null;
        }
        SmDeviceVo device = smDeviceMapper.selectByDeviceNo(deviceSn);
        if (device != null) {
            deviceAssembler.assembleSuitList(Collections.singletonList(device));
        }
        return device;
    }

    /**
     * 根据设备编号查询
     *
     * @param deviceNo
     */
    @Override
    public SmDeviceVo selectByDeviceNo(String deviceNo) {
        return smDeviceMapper.selectByDeviceNo(deviceNo);
    }

    /**
     * 初始设备录入
     *
     * @param smDevice 设备信息
     */
    @Override
    public int addInitDevice(SmDevice smDevice) {
        if (smDevice == null) {
            return 0;
        }
        try {
            Integer result = transactionTemplate.execute(status -> {
                SmDeviceVo byMac = selectByMac(smDevice.getMac());
                if (byMac == null) {
                    smDevice.setCreateTime(DateUtils.getNowDate());
                    return smDeviceMapper.insertSmDevice(smDevice);
                }
                return 0;
            });
            return result == null ? 0 : result;
        } catch (Exception e) {
            log.error("add init device error: {}", e.getMessage());
            return 0;
        }
    }

    @Override
    @Transactional
    public boolean unbindStore(Long deviceId) {
        SmDeviceVo device = smDeviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        ServiceUtil.assertion(DeviceStatus.USING.getStatus().equals(device.getStatus()), "设备正在使用中，无法解除绑定");

        int updateCount = smDeviceMapper.unbindStore(deviceId);
        ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");
        return true;
    }

    /**
     * 设备绑定SN码
     *
     * @param deviceId 设备ID
     * @param sn       SN
     */
    @Override
    public int bindSn(Long deviceId, String sn) {
        Integer result = transactionTemplate.execute(status -> {
            int i = smDeviceMapper.bindSn(deviceId, sn);
            ServiceUtil.assertion(i != 1, "设备SN已发生变化，请刷新后重试");
            return i;
        });
        return result == null ? 0 : result;
    }
}
