package com.ruoyi.ss.user.service.impl;

import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.user.domain.SmUserQuery;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.mapper.SmUserMapper;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 普通用户信息Service业务层处理
 *
 * @author qzz
 * @date 2024-01-24
 */
@Service
public class SmUserServiceImpl implements ISmUserService
{
    @Autowired
    private SmUserMapper smUserMapper;

    /**
     * 查询普通用户信息
     *
     * @param userId 普通用户信息主键
     * @return 普通用户信息
     */
    @Override
    public SmUserVo selectSmUserByUserId(Long userId)
    {
        return smUserMapper.selectSmUserByUserId(userId);
    }

    /**
     * 查询普通用户信息列表
     *
     * @param dto 查询条件
     * @return 普通用户信息
     */
    @Override
    public List<SmUserVo> selectSmUserList(SmUserQuery dto)
    {
        List<SmUserVo> list = smUserMapper.selectSmUserList(dto);
        desensitization(list);
        return list;
    }

    /**
     * 数据脱敏
     * @param list
     */
    @Override
    public void desensitization(List<SmUserVo> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        list.forEach(item -> {
            item.setIdentityCard(null);
            item.setPassword(null);
            item.setPayPassword(null);
        });
    }

    @Override
    public Integer selectCount(SmUserQuery dto) {
        return smUserMapper.selectCount(dto);
    }

    @Override
    public SmUserVo selectUserByPhone(String phoneNumber) {
        if (phoneNumber == null) {
            return null;
        }
        SmUserQuery dto = new SmUserQuery();
        dto.setPhonenumber(phoneNumber);
        List<SmUserVo> list = smUserMapper.selectSmUserList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public boolean changeType(Long userId, UserType userType) {
        // 修改数据库中的数据
        SmUser smUser = new SmUser();
        smUser.setUserId(userId);
        smUserMapper.updateSmUser(smUser);

        return true;
    }

    @Override
    @Transactional
    public void addBalance(Long userId, BigDecimal amount) {
        ServiceUtil.assertion(BigDecimal.ZERO.compareTo(amount) > 0, "增加的金额需要大于0");
        int updateCount = smUserMapper.addBalance(userId, amount);
        ServiceUtil.assertion(updateCount != 1, "增加用户余额失败");
    }

    @Override
    @Transactional
    public void subtractBalance(Long userId, BigDecimal amount) {
        ServiceUtil.assertion(BigDecimal.ZERO.compareTo(amount) > 0, "减少的金额需要大于0");
        int updateCount = smUserMapper.subtractBalance(userId, amount);
        ServiceUtil.assertion(updateCount != 1, "减少用户余额失败：用户不存在或余额不足");
    }


    @Override
    @Transactional
    public void updateWxOpenId(Long userId, String openId) {
        // 判断该openId是否已经被绑定
        SmUserVo repeatUser = smUserMapper.selectSmUserByWxOpenId(openId);
        ServiceUtil.assertion(repeatUser != null && !Objects.equals(repeatUser.getUserId(), userId), "该微信已被绑定");

        // 修改openId
        SmUser user = new SmUser();
        user.setUserId(userId);
        user.setWxOpenId(openId);
        smUserMapper.updateSmUser(user);
    }

    @Override
    public SmUserVo selectSimpleById(Long userId) {
        return smUserMapper.selectSimpleById(userId);
    }

    @Override
    public boolean logicDel(List<Long> userIds) {
        this.validatePreLogicDel(userIds);
        return smUserMapper.logicDelByIds(userIds);
    }

    @Override
    public boolean isExistUsers(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return true;
        }
        Map<Long, SmUserVo> userMap = this.selectMapByUserIds(userIds);
        for (Long userId : userIds) {
            SmUserVo user = userMap.get(userId);
            if (user == null) {
                return false;
            }
        }
        return true;
    }

    @Override
    public SmUserVo selectSmUserByWxOpenId(String openid) {
        return smUserMapper.selectSmUserByWxOpenId(openid);
    }

    @Override
    @Transactional
    public boolean updatePassword(Long userId, String password) {
        ServiceUtil.assertion(userId == null || password == null, "参数错误：用户id或密码为空");
        SmUser user = new SmUser();
        user.setUserId(userId);
        user.setPassword(SecurityUtils.encryptPassword(password));
        int updateCount = smUserMapper.updateSmUser(user);
        ServiceUtil.assertion(updateCount != 1, "修改密码失败，请刷新后重试");
        return true;
    }

    /**
     * ids查询列表
     *
     * @param ids
     */
    @Override
    public List<SmUserVo> selectByIds(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        SmUserQuery query = new SmUserQuery();
        query.setUserIds(ids);
        return this.selectSmUserList(query);
    }

    /**
     * 将用户设置为商家
     *
     * @param userId
     */
    @Override
    public int setMch(Long userId) {
        if (userId == null) {
            return 0;
        }
        SmUser user = new SmUser();
        user.setUserId(userId);
        user.setIsMch(true);
        return smUserMapper.updateSmUser(user);
    }

    /**
     * 逻辑删除前校验
     * @param userIds
     */
    private void validatePreLogicDel(List<Long> userIds) {
        ServiceUtil.assertion(!CollectionUtils.isEmpty(userIds), "用户id不能为空");
        ServiceUtil.assertion(!isExistUsers(userIds), "用户不存在");
    }

    /**
     * 查询用户映射表
     * @param userIds
     */
    private Map<Long, SmUserVo> selectMapByUserIds(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyMap();
        }
        SmUserQuery dto = new SmUserQuery();
        dto.setUserIds(userIds);
        List<SmUserVo> list = smUserMapper.selectSmUserList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(SmUserVo::getUserId, item -> item));
    }

    /**
     * 新增普通用户信息
     *
     * @param smUser 普通用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertSmUser(SmUser smUser)
    {
        this.validate(smUser, false);
        smUser.setCreateTime(DateUtils.getNowDate());
        return smUserMapper.insertSmUser(smUser);
    }

    /**
     * 插入、更新校验
     * @param data 数据
     * @param update 是否更新
     */
    private void validate(SmUser data, boolean update) {
        if (update) {
            SmUserVo user = selectSmUserByUserId(data.getUserId());
            ServiceUtil.assertion(user == null, "用户不存在");
        }

        // 判断是否重复注册
        SmUserVo repeatPhone = selectUserByPhone(data.getPhonenumber());
        ServiceUtil.assertion(repeatPhone != null && !Objects.equals(repeatPhone.getUserId(), data.getUserId()), "用户手机号重复");

        SmUserVo repeatIdCard = selectUserByIdCard(data.getIdentityCard());
        ServiceUtil.assertion(repeatIdCard != null && !Objects.equals(repeatIdCard.getUserId(), data.getUserId()), "用户身份证号重复");

        SmUserVo repeatWxOpenId = selectUserByWxOpenId(data.getWxOpenId());
        ServiceUtil.assertion(repeatWxOpenId != null && repeatWxOpenId.getWxOpenId() != null &&
                !Objects.equals(repeatWxOpenId.getUserId(), data.getUserId()), "用户微信openId重复");

    }

    // 根据微信openId查询信息
    private SmUserVo selectUserByWxOpenId(String wxOpenId) {
        if (wxOpenId == null) {
            return null;
        }
        return smUserMapper.selectSmUserByWxOpenId(wxOpenId);
    }

    // 根据身份证号查询用户信息
    private SmUserVo selectUserByIdCard(String idCard) {
        if (idCard == null) {
            return null;
        }
        SmUserQuery dto = new SmUserQuery();
        dto.setIdentityCard(idCard);
        List<SmUserVo> list = smUserMapper.selectSmUserList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 修改普通用户信息
     *
     * @param smUser 普通用户信息
     * @return 结果
     */
    @Override
    public int updateSmUser(SmUser smUser)
    {
        this.validate(smUser, true);
        smUser.setUpdateTime(DateUtils.getNowDate());
        return smUserMapper.updateSmUser(smUser);
    }

    /**
     * 批量删除普通用户信息
     *
     * @param userIds 需要删除的普通用户信息主键
     * @return 结果
     */
    @Override
    public int deleteSmUserByUserIds(Long[] userIds)
    {
        return smUserMapper.deleteSmUserByUserIds(userIds);
    }

    /**
     * 删除普通用户信息信息
     *
     * @param userId 普通用户信息主键
     * @return 结果
     */
    @Override
    public int deleteSmUserByUserId(Long userId)
    {
        return smUserMapper.deleteSmUserByUserId(userId);
    }

    /**
     * 根据用户名获取普通用户信息
     * @param username
     * @return
     */
    @Override
    public SmUserVo selectUserByUserName(String username) {
        if (username == null) {
            return null;
        }
        SmUserQuery smUser = new SmUserQuery();
        smUser.setUserName(username);
        List<SmUserVo> list = smUserMapper.selectSmUserList(smUser);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }
}
